# -*- coding: utf-8 -*-
# Generated by Django 1.11.1 on 2017-11-28 03:40
from __future__ import unicode_literals

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('contenttypes', '0002_remove_content_type_name'),
        ('gl', '0002_transaction_bmo_id'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='transaction',
            name='company',
        ),
        migrations.RemoveField(
            model_name='transaction',
            name='content_type',
        ),
        migrations.RemoveField(
            model_name='tranline',
            name='transaction',
        ),
        migrations.AddField(
            model_name='tranline',
            name='bmo_id',
            field=models.CharField(blank=True, max_length=100, null=True),
        ),
        migrations.AddField(
            model_name='tranline',
            name='comment',
            field=models.CharField(blank=True, max_length=100, null=True),
        ),
        migrations.AddField(
            model_name='tranline',
            name='company',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, to='gl.Company'),
        ),
        migrations.AddField(
            model_name='tranline',
            name='content_type',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, to='contenttypes.ContentType'),
        ),
        migrations.AddField(
            model_name='tranline',
            name='date_end',
            field=models.DateField(blank=True, db_index=True, null=True),
        ),
        migrations.AddField(
            model_name='tranline',
            name='object_id',
            field=models.PositiveIntegerField(blank=True, null=True),
        ),
        migrations.AlterField(
            model_name='account',
            name='display_name',
            field=models.CharField(blank=True, help_text='the name you would use on a report title', max_length=100),
        ),
        migrations.AlterField(
            model_name='account',
            name='id',
            field=models.CharField(help_text='Unique short code or number', max_length=20, primary_key=True, serialize=False),
        ),
        migrations.AlterField(
            model_name='account',
            name='path',
            field=models.CharField(db_index=True, help_text='dotted-path reference to show location on balance sheet', max_length=100, unique=True),
        ),
        migrations.AlterField(
            model_name='account',
            name='role',
            field=models.CharField(choices=[('asset', 'asset'), ('liability', 'liability'), ('income', 'income'), ('expense', 'expense'), ('capital', 'capital')], help_text='used in P&L queries and for picklists', max_length=10),
        ),
        migrations.AlterField(
            model_name='company',
            name='cmpy_type',
            field=models.CharField(choices=[('ALO', 'Standalone'), ('CON', 'Consolidation')], max_length=10),
        ),
        migrations.AlterField(
            model_name='tranline',
            name='account',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, to='gl.Account'),
        ),
        migrations.DeleteModel(
            name='Transaction',
        ),
    ]
