# -*- coding: utf-8 -*-
# Generated by Django 1.11.1 on 2017-11-27 01:10
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('common', '0003_address_label'),
    ]

    operations = [
        migrations.AlterField(
            model_name='taskmonitor',
            name='task_state',
            field=models.CharField(choices=[('started', 'started'), ('failed', 'failed'), ('succeeded', 'succeeded')], max_length=200, null=True),
        ),
    ]
