# -*- coding: utf-8 -*-
# Generated by Django 1.11.1 on 2017-12-16 23:04
from __future__ import unicode_literals

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('common', '0004_auto_20171126_2010'),
    ]

    operations = [
        migrations.AlterModelTable(
            name='address',
            table='common_address',
        ),
        migrations.AlterModelTable(
            name='log',
            table='common_log',
        ),
        migrations.AlterModelTable(
            name='taskmonitor',
            table='common_taskmonitor',
        ),
    ]
