# -*- coding: utf-8 -*-
# Generated by Django 1.11.1 on 2017-11-27 01:10
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('cal', '0001_initial'),
    ]

    operations = [
        migrations.AlterField(
            model_name='month',
            name='id',
            field=models.CharField(help_text='e.g. "2009M04"', max_length=7, primary_key=True, serialize=False),
        ),
        migrations.AlterField(
            model_name='quarter',
            name='id',
            field=models.CharField(help_text='e.g. "2009Q1"', max_length=6, primary_key=True, serialize=False),
        ),
        migrations.AlterField(
            model_name='year',
            name='id',
            field=models.IntegerField(help_text='4 digit year e.g. 2009', primary_key=True, serialize=False),
        ),
    ]
