# -*- coding: utf-8 -*-
# Generated by Django 1.11.1 on 2017-12-21 02:38
from __future__ import unicode_literals

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('gl', '0004_tranline_date'),
    ]

    operations = [
        migrations.AlterField(
            model_name='tranline',
            name='counterparty',
            field=models.ForeignKey(blank=True, max_length=50, null=True, on_delete=django.db.models.deletion.CASCADE, to='gl.Counterparty'),
        ),
    ]
