# Generated by Django 2.0.5 on 2018-06-16 20:43

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('gl', '0009_auto_20180607_1718'),
    ]

    operations = [
        migrations.AddField(
            model_name='tranline',
            name='closing_entry',
            field=models.BooleanField(default=False),
        ),
    ]
