# Generated by Django 2.0.5 on 2018-06-07 21:18

import django.core.validators
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('gl', '0008_auto_20180511_1529'),
    ]

    operations = [
        migrations.AlterField(
            model_name='company',
            name='label',
            field=models.CharField(default='TBD', max_length=20),
        ),
        migrations.AlterField(
            model_name='counterparty',
            name='id',
            field=models.CharField(max_length=30, primary_key=True, serialize=False, validators=[django.core.validators.RegexValidator('^[0-9a-zA-Z]*$', 'Only alphanumeric characters are allowed. In particular.. no spaces.')]),
        ),
        migrations.AlterField(
            model_name='counterparty',
            name='label',
            field=models.CharField(default='TBD', max_length=20),
        ),
    ]
