# -*- coding: utf-8 -*-
# Generated by Django 1.11.1 on 2017-12-21 12:36
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('gl', '0005_auto_20171220_2138'),
    ]

    operations = [
        migrations.AlterField(
            model_name='counterparty',
            name='id',
            field=models.CharField(max_length=30, primary_key=True, serialize=False),
        ),
    ]
