# -*- coding: utf-8 -*-
# Generated by Django 1.11.1 on 2017-12-31 22:16
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('common', '0005_auto_20171216_1804'),
    ]

    operations = [
        migrations.AddField(
            model_name='log',
            name='corrId',
            field=models.CharField(blank=True, max_length=20, null=True),
        ),
    ]
