# -*- coding: utf-8 -*-
# Generated by Django 1.11.1 on 2017-12-16 23:04

from __future__ import unicode_literals

import datetime
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('gl', '0003_auto_20171127_2240'),
    ]

    operations = [
        migrations.AddField(
            model_name='tranline',
            name='date',
            field=models.DateField(db_index=True, default=datetime.date(2015, 1, 1)),
            preserve_default=False,
        ),
    ]
